package com.repro

import android.content.Context
import android.content.SharedPreferences
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.core.content.edit
import com.tfcporciuncula.flow.FlowSharedPreferences
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposables
import io.reactivex.schedulers.Schedulers
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.Job
import kotlinx.coroutines.cancel
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.reactive.asPublisher
import kotlinx.coroutines.rx2.asObservable
import kotlin.concurrent.thread

private const val KEY_REPRO = "key_repro"

@ExperimentalCoroutinesApi
class MainActivity : AppCompatActivity() {

    private var disposable = Disposables.disposed()

    private lateinit var preferences: SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        preferences = getPreferences(Context.MODE_PRIVATE)
        disposable = FlowSharedPreferences(preferences)
            .getBoolean(KEY_REPRO)
            .asFlow()
            .asObservable()
            .subscribe {
                Log.d("MainActivity", "Preferences value is $it")
            }
    }

    override fun onDestroy() {
        disposable.dispose()
        super.onDestroy()
    }

    fun createPreference(view: View) {
        preferences.edit {
            putBoolean(KEY_REPRO, true)
        }
    }

    fun attemptReproCrash(view: View) {
        thread {
            // If a Thread.sleep is placed here, the crash no longer repros.
            preferences.edit {
                remove(KEY_REPRO)
            }
        }
        disposable.dispose()
    }
}
